from zavod.sinks.common import Sink
from zavod.sinks.json_entity import JSONEntitySink
from zavod.sinks.json_statement import JSONStatementSink
from zavod.sinks.pack_statement import PackStatementSink

__all__ = [
    "Sink",
    "JSONEntitySink",
    "JSONStatementSink",
    "PackStatementSink",
]
