from zavod.parse.addresses import make_address, format_address
from zavod.parse.xml import remove_namespace
from zavod.parse.names import make_name, apply_name

__all__ = [
    "make_address",
    "format_address",
    "remove_namespace",
    "make_name",
    "apply_name",
]
